;**********************************************
;*  GBClock V1.0 - Talking Clock for GameBoy  *
;**********************************************
; By Jeff Frohwein

; Started 29-Nov-97
; Last edit 29-Nov-97

; Compiled with RGBASM V1.10

; All include files not distributed with this file
; may be found on the Code Libraries web page.
; http://hiwaay.net/~jfrohwei/gameboy/libs.html

        INCLUDE "hardware.inc"
        INCLUDE "ibmpc1.inc"

;Version 1.5 or later of 'hardware.inc' is required.
        rev_Check_hardware_inc 1.5
;Version 1.1 or later of 'ibmpc1.inc' is required.
        rev_Check_hardware_inc 1.1

; Ram variables
        SECTION "Low Ram",BSS
ClkHours        DB                      ; Holds hour count (range 0-23)
ClkTenMins      DB                      ; Holde tens of minutes (range 0-5)
ClkMins         DB                      ; Holds minutes (range 0-9)
ClkSecs         DB                      ; Holds seconds (range 1-60)
Clk60thSecs     DB                      ; Holds 1/60ths of a second (range 1-60)

TempHours       DB                      ; Holds temp hour count (range 0-23)
TempTenMins     DB                      ; Holde temp tens of minutes (range 0-5)
TempMins        DB                      ; Holds temp minutes (range 0-9)

OldClkMins      DB                      ; Holds old minutes
VBlankDone      DB                      ; VBlank done flag
CtrlCnt         DB                      ; Counter used by controls

; Defines
CLK_POSITION    EQU     $9800+6+(SCRN_VY_B*7) ; Location of first digit on screen
BOGUS_VALUE     EQU     99                    ; Bogus value for minutes (causes time redraw)

        SECTION "Org $0",HOME
        ret

; VBlank Interrupt - For updating clock registers

        SECTION "Org $40",HOME[$40]
        push    af
        ld      a,1
        ld      [VBlankDone],a

        ld      a,[Clk60thSecs]
        dec     a                       ;Are we ready to increment seconds?
        jr      nz,.cux0                ;Not yet

        ld      a,60
        ld      [Clk60thSecs],a

        ld      a,[ClkSecs]
        dec     a                       ;Are we ready to increment minutes?
        jr      nz,.cux1                ;Not yet

        ld      a,60
        ld      [ClkSecs],a

        ld      a,[ClkMins]
        inc     a
        cp      10                      ;Has single-digit minutes overflowed?
        jr      c,.cux2                 ;Not yet

        xor     a
        ld      [ClkMins],a

        ld      a,[ClkTenMins]
        inc     a
        cp      6                       ;Has double-digit minutes overflowed?
        jr      c,.cux3                 ;Not yet

        xor     a
        ld      [ClkTenMins],a

        ld      a,[ClkHours]
        inc     a
        cp      24                      ;Has Hours overflowed?
        jr      c,.cux4                  ;Not yet

        xor     a
        ld      [ClkHours],a

        pop     af
        reti

.cux0:  ld      [Clk60thSecs],a
        pop     af
        reti

.cux1:  ld      [ClkSecs],a
        pop     af
        reti

.cux2:  ld      [ClkMins],a
        pop     af
        reti

.cux3:  ld      [ClkTenMins],a
        pop     af
        reti

.cux4:  ld      [ClkHours],a
        pop     af
        reti

        DB      "Date 971129, Jeff Frohwein"           ; Date stamping info for
                                        ; reference use only
        SECTION "Org $100",HOME[$100]

;*** Beginning of rom execution point ***

        nop
        jp      begin

        NINTENDO_LOGO                   ; Nintendo graphic logo

;Rom Header Info

;    0123456789ABCDEF
 DB "GB CLOCK V1.0",0,0,0    ; Cart name   16bytes
 DB 0,0,0                     ; Not used
 DB CART_ROM                  ; Cart type
 DB CART_ROM_256K             ; ROM Size (in bits)
 DB CART_RAM_NONE             ; RAM Size (in bits)
 DB 0,0                       ; Maker ID
 DB 0                         ; Version     =0
 DB 0                         ; Complement check (important)
 DW 0                         ; Checksum (not important)

        INCLUDE "memory1.asm"
        INCLUDE "keypad.asm"

;Version 1.1 or later of 'memory1.asm' is required.
        rev_Check_memory1_asm 1.1
;Version 1.2 or later of 'keypad.asm' is required.
        rev_Check_keypad_asm 1.2

        SECTION "Main Code",HOME
begin:
        call    Initialize      ; Initialize everything
Main:
        call    PowerDown       ; Power down until VBlank
        call    DisplayTime
        call    Controls
        jr      Main

; Speak the time
SpeakTime:
        di
        ld      a,[ClkHours]            ; Get a record of the current time
        ld      [TempHours],a           ; and speak that time. The time
        ld      a,[ClkTenMins]          ; registers can change while speaking
        ld      [TempTenMins],a         ; is taking place so we must use a copy
        ld      a,[ClkMins]             ; of them instead of using them.
        ld      [TempMins],a
        ei

        ld      a,[TempHours]
        cp      12
        jr      c,.spk1
        sub     12
.spk1:
        call    SpeakNum

        ld      a,[TempTenMins]
        or      a               ;is tens of minutes = 0 ?
        jr      nz,.spk2        ;no

        ld      a,[TempMins]
        or      a               ;is mins = 0?
        jr      z,.spkx         ;yes

        ld      a,BANK(Oh)      ;Speak "Oh"
        ld      hl,Oh
        call    SpeakPhrase

        jr      .spkm

.spk2:
        dec     a               ;is it 1X minutes ?
        jr      nz,.spk3        ;no

        call    Speak1Xmins     ;Speak minutes from 10-19
        jr      .spkx
.spk3:
        call    Speak29Xmins    ;Speak minutes from 20-59

        ld      a,[TempMins]
        or      a               ;is mins = 0?
        jr      z,.spkx         ;yes

.spkm:
        ld      a,[TempMins]
        call    SpeakNum        ;Speak Mins
.spkx:
        ld      a,[TempHours]
        cp      12              ;Is it AM?

        ld      a,BANK(LetterA) ;Speak "A"
        ld      hl,LetterA

        jr      c,.spky         ;Yes

        ld      a,BANK(LetterP) ;Speak "P"
        ld      hl,LetterP
.spky:
        call    SpeakPhrase

        ld      a,BANK(LetterM) ;Speak "M"
        ld      hl,LetterM
        call    SpeakMedium
        ret

Speak1Xmins:
        ld      a,[TempMins]

        ld      b,BANK(Ten)
        ld      hl,Ten

        or      a               ;is it ten?
        jr      z,.Speak1Xx     ;yes

        ld      b,BANK(Eleven)
        ld      hl,Eleven

        dec     a               ;is it eleven?
        jr      z,.Speak1Xx     ;yes

        ld      b,BANK(Twelve)
        ld      hl,Twelve

        dec     a               ;is it twelve?
        jr      z,.Speak1Xx     ;yes

        ld      b,a
        add     a               ;a=a*2
        add     b               ;a=a*3
        ld      d,0
        ld      e,a
        ld      hl,TeenTable
        add     hl,de

        ld      b,[hl]
        inc     hl
        ld      a,[hl+]
        ld      h,[hl]
        ld      l,a

        ld      a,b
        call    SpeakPhrase     ;Speak the pre-teens

        ld      a,[TempMins]     ;Both 13 & 15 need small
        cp      3               ;delay in the middle
        jr      z,.Speak1Xd
        cp      5
        jr      nz,.Speak1Xs
.Speak1Xd:
        call    SmallDelay
.Speak1Xs:
        ld      b,BANK(Teen)    ;Speak "Teen"
        ld      hl,Teen
.Speak1Xx:
        ld      a,b
        call    SpeakMedium
        ret

TeenTable:
        DB      BANK(Twen)
         DW      Twen
        DB      BANK(Thir)
         DW      Thir
        DB      BANK(Fou)
         DW      Fou
        DB      BANK(Fif)
         DW      Fif
        DB      BANK(Six)
         DW      Six
        DB      BANK(Seven)
         DW      Seven
        DB      BANK(Eight)
         DW      Eight
        DB      BANK(Nin)
         DW      Nin

Speak29Xmins:
        ld      a,[TempTenMins]
        dec     a
        dec     a

        ld      b,a
        add     a               ;a=a*2
        add     b               ;a=a*3
        ld      d,0
        ld      e,a
        ld      hl,TeenTable
        add     hl,de

        ld      b,[hl]
        inc     hl
        ld      a,[hl+]
        ld      h,[hl]
        ld      l,a

        ld      a,b
        call    SpeakPhrase     ;Speak the pre-teens

        ld      a,[TempTenMins]     ;Both 30 & 50 need small
        cp      3                   ;delay in the middle
        jr      z,Speak29Xd
        cp      5
        jr      nz,Speak29Xs
Speak29Xd:
        call    SmallDelay
Speak29Xs:

        ld      a,BANK(LetterT) ;Speak "T"
        ld      hl,LetterT
        call    SpeakMedium
        ret

; Speak digit in A
SpeakNum:
        ld      b,a
        add     a       ; a = a * 2
        add     b       ; a = a * 3
        ld      d,0
        ld      e,a
        ld      hl,HourTable
        add     hl,de

        ld      b,[hl]
        inc     hl
        ld      a,[hl+]
        ld      h,[hl]
        ld      l,a

        ld      a,b
        call    SpeakMedium     ;Speak Hours
        ret

; Speak phrase in bank A at address HL
SpeakPhrase:
        call    SetBank
        call    snd_Sample1
        ret

HourTable:
        DB      BANK(Twelve)
         DW      Twelve
        DB      BANK(One)
         DW      One
        DB      BANK(Two)
         DW      Two
        DB      BANK(Three)
         DW      Three
        DB      BANK(Four)
         DW      Four
        DB      BANK(Five)
         DW      Five
        DB      BANK(Six)
         DW      Six
        DB      BANK(Seven)
         DW      Seven
        DB      BANK(Eight)
         DW      Eight
        DB      BANK(Nine)
         DW      Nine
        DB      BANK(Ten)
         DW      Ten
        DB      BANK(Eleven)
         DW      Eleven

;Set ROM Bank
SetBank:
        ld      [$2000],a
        ret

;Process controls
Controls:
        ld      a,[CtrlCnt]
        dec     a
        jr      nz,Cont1
        ld      a,20
Cont1:  ld      [CtrlCnt],a

        call    pad_Read
        bit     PADB_START,a
        jr      z,Cont2

; Increment minutes

        ld      a,[CtrlCnt]
        dec     a
        jr      nz,Cont2

        ld      a,[ClkMins]
        inc     a
        ld      [ClkMins],a
        cp      10
        jr      c,Cont2

        xor     a
        ld      [ClkMins],a

        ld      a,[ClkTenMins]
        inc     a
        ld      [ClkTenMins],a
        cp      6
        jr      c,Cont2

        xor     a
        ld      [ClkTenMins],a

Cont2:
        call    pad_Read
        bit     PADB_SELECT,a
        jr      z,Cont4

; Increment hours

        ld      a,[CtrlCnt]
        dec     a
        jr      nz,Cont4

        ld      a,[ClkHours]
        inc     a
        ld      [ClkHours],a
        cp      24
        jr      c,Cont3

        xor     a
        ld      [ClkHours],a
Cont3:
        ld      a,BOGUS_VALUE
        ld      [OldClkMins],a          ;force time redraw

Cont4:
        call    pad_Read
        bit     PADB_A,a
        jr      z,Cont5

        call    SpeakTime

Cont5:
        call    pad_Read
        bit     PADB_B,a
        jr      z,Cont6

        call    SpeakTime
Cont6:
        ret

; Power on initialize
Initialize:
        di
                  ; The stack initializes to $FFFE

        ld      a,0
        ldh     [rIE],a         ;disable interrupts

; Must be in VBL before turning the screen off.
WaitVBL:
        ldh    a,[rLY]         ; $ff44=LCDC Y-Pos
        cp     $90             ; $90 and bigger = in VBL
        jr     c,WaitVBL       ; Loop until it is $90 or >

        ld     a,LCDCF_OFF
        ldh    [rLCDC],a        ; Turn screen off for memory copy

        ld      a,$e4
        ldh     [rBGP],a        ; Setup default background colors

        ld      hl,TileData
        ld      de,$8000
        ld      bc,8*128
        call    mem_CopyMono    ; Copy tile data to memory

        ld      a,$20           ; Clear tile map memory
        ld      hl,$9800
        ld      bc,SCRN_VX_B * SCRN_VY_B
        call    mem_Set

        call    DrawScreen              ;Draw Box & text on screen

        xor     a                       ;Clear time variables
        ld      [ClkHours],a
        ld      [ClkTenMins],a
        ld      [ClkMins],a
        ld      [ClkSecs],a
        ld      a,60
        ld      [Clk60thSecs],a
        ld      [ClkSecs],a

        ld      a,BOGUS_VALUE
        ld      [OldClkMins],a          ;load with bogus value to force time redraw

        ld      a,LCDCF_ON|LCDCF_BG8000|LCDCF_BG9800|LCDCF_BGON|LCDCF_OBJ16|LCDCF_OBJOFF
        ldh     [rLCDC],a       ; Turn screen on

        ld      a,1
        ldh     [rIE],a         ;enable V-Blank interrupt

        ei

        ret

; Display Time
DisplayTime:
        ld      a,[OldClkMins]
        ld      b,a
        ld      a,[ClkMins]
        cp      b               ; Has minutes changed recently?
        ret     z               ; no, don't update screen
        ld      [OldClkMins],a

        ld      a,[ClkHours]
        cp      12              ; Is time between 12 - 23 hours?
        jr      c,dt1
        sub     12              ; yes, make it span from 0-11.
dt1:
        or      a               ; Is hours 0?
        jr      nz,dt2
        ld      a,12            ; yes, display that as 12 instead.
dt2:
        ld      b," "
        cp      10              ; Is hours 10 or greater?
        jr      c,dt3           ; no
        ld      b,"1"
        sub     10
dt3:
        ld      hl,CLK_POSITION

        push    af
        ld      a,b
        call    DrawDigit       ; Display tens of hours
        pop     af

        add     "0"
        call    DrawDigit       ; Display Hours

        ld      a,":"
        call    DrawDigit

        ld      a,[ClkTenMins]  ; Display tens of minutes
        add     "0"
        call    DrawDigit

        ld      a,[ClkMins]     ; Display minutes
        add     "0"
        call    DrawDigit

        ld      a," "
        call    DrawDigit

        ld      a,[ClkHours]    ; Display AM or PM
        cp      12
        ld      a,"A"
        jr      c,dt4
        ld      a,"P"
dt4:    call    DrawDigit

        ld      a,"M"
        call    DrawDigit
        ret

;Write A to screen memory [HL] when VRAM is ready
DrawDigit:
        push    af
        lcd_WaitVRAM
        pop     af
        ld      [hl],a
        inc     hl
        ret

; Draw screen & commands
DrawScreen:
        ld      hl,Title       ; Draw title
        ld      de,$9800+4+(SCRN_VY_B*2)
        ld      bc,12
        call    mem_Copy

        ld      hl,Pic1        ; Draw box
        ld      de,$9800+3+(SCRN_VY_B*5)
        ld      bc,14
        call    mem_Copy

        ld      hl,Pic2        ; Draw box
        ld      de,$9800+3+(SCRN_VY_B*6)
        ld      bc,14
        call    mem_Copy

        ld      hl,Pic2        ; Draw box
        ld      de,$9800+3+(SCRN_VY_B*7)
        ld      bc,14
        call    mem_Copy

        ld      hl,Pic2        ; Draw box
        ld      de,$9800+3+(SCRN_VY_B*8)
        ld      bc,14
        call    mem_Copy

        ld      hl,Pic1        ; Draw box
        ld      de,$9800+3+(SCRN_VY_B*9)
        ld      bc,14
        call    mem_Copy

        ld      hl,Cmnd1
        ld      de,$9800+1+(SCRN_VY_B*12)
        ld      bc,18
        call    mem_Copy

        ld      hl,Cmnd2
        ld      de,$9800+1+(SCRN_VY_B*13)
        ld      bc,17
        call    mem_Copy

        ld      hl,Cmnd3
        ld      de,$9800+1+(SCRN_VY_B*14)
        ld      bc,14
        call    mem_Copy

        ret

SignOn: DB      "12:00 AM"

Title:  DB      "GBClock V1.0"
Pic1:   DB      $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
Pic2:   DB      $00,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$00
Cmnd1:  DB      "SELECT = Set Hours"
Cmnd2:  DB      "START  = Set Mins"
Cmnd3:  DB      "A or B = Speak"

; Halt until VBlank

PowerDown:
        halt

        ld      a,[VBlankDone]
        rra
        jr      nc,PowerDown

        xor     a
        ld      [VBlankDone],a
        ret

; * Speak phrase then do a medium delay *
SpeakMedium:
        call    SpeakPhrase

; * Medium delay between words *
MediumDelay:
        ld      b,4
        jr      delay1

;* Small delay between words *
SmallDelay:
        ld      b,1

delay1: ld      hl,$2000
delay2  dec     hl
        ld      a,h
        or      l
        jr      nz,delay2

        dec     b
        jr      nz,delay1

        ret

; Library Includes

        INCLUDE "sample1.asm"

; Font library
TileData:
        chr_IBMPC1      1,4

;Version 1.2 or later of 'sample1.asm' is required.
        rev_Check_sample1_asm 1.2

; Speech libraries
Zero:
        INCLUDE "tlke0.asm"
One:
        INCLUDE "tlke1.asm"
Two:
        INCLUDE "tlke2.asm"
Three:
        INCLUDE "tlke3.asm"


        SECTION "Bank 1",CODE
Fou:
        DW      $78
Four:
        INCLUDE "tlke4.asm"
Five:
        INCLUDE "tlke5.asm"
Six:
        INCLUDE "tlke6.asm"
Seven:
        INCLUDE "tlke7.asm"
Eight:
        INCLUDE "tlke8.asm"
Nin:
        DW      $70
Nine:
        INCLUDE "tlke9.asm"


        SECTION "Bank 2",CODE
Ten:
        INCLUDE "tlke10.asm"
Eleven:
        INCLUDE "tlke11.asm"
Twelve:
        INCLUDE "tlke12.asm"
Oh:
        INCLUDE "tlkeoh.asm"
LetterA:
        INCLUDE "tlkea.asm"
LetterP:
        INCLUDE "tlkep.asm"
LetterM:
        INCLUDE "tlkem.asm"


        SECTION "Bank 3",CODE
LetterT:
        INCLUDE "tlket.asm"
Teen:
        INCLUDE "tlketeen.asm"
Thir:
        INCLUDE "tlkethir.asm"
Fif:
        INCLUDE "tlkefif.asm"
Twen:
        INCLUDE "tlketwen.asm"
;*
